@@ -53,13 +53,10 @@
 #include "mcd-signals-marshal.h"
 #include "mcd-dispatcher.h"
 #include "mcd-dispatcher-context.h"
-#include "mcd-account-compat.h"
 #include "mcd-connection.h"
 #include "mcd-misc.h"
 #include "mcd-service.h"
 
-#include <libmcclient/mc-errors.h>
-
 /* DBus service specifics */
 #define MISSION_CONTROL_DBUS_SERVICE "org.freedesktop.Telepathy.MissionControl5"
 
@@ -90,8 +87,8 @@ mcd_service_obtain_bus_name (McdService * obj)
                                       MISSION_CONTROL_DBUS_SERVICE,
                                       TRUE /* idempotent */, &error))
     {
-        g_error ("Failed registering '%s' service: %s",
-                 MISSION_CONTROL_DBUS_SERVICE, error->message);
+        g_warning ("Failed registering '%s' service: %s",
+                   MISSION_CONTROL_DBUS_SERVICE, error->message);
         g_error_free (error);
         exit (1);
     }
